@@ -58,9 +58,8 @@ each proc sub should take the following argv
   cxx impl: use this as the cxx impl instead of the default
   returns alt type: the constructor returns some type other than
     the object from which it is a member of
-  no native: do not attempt to create a native implementation
-  treat as object: treat as an object rather than a pointer
-  no conv: do not converted an encoded string
+  no native: do not attemt to create a native implementation
+  treat as object: treat as a object rather than a pointer
 
 The %info structure is initialized as follows:
 
@@ -79,10 +78,10 @@ The %info structure is initialized as follows:
     options => ['strip', 'prefix', 'c impl headers'],
     groups => undef},
   group => {
-    # a group is a collection of objects which should be grouped together
+    # a group is a colection of objects which should be grouped together
     # this generally means they will be in the same source file
-    options => ['no native'],
-    groups => ['enum', 'struct', 'union', 'func', 'class', 'errors']},
+    options => ['no native', 'no impl'],
+    groups => ['enum', 'struct', 'union', 'func', 'callback', 'class', 'errors']},
   enum => {
     # basic C enum
     options => ['desc', 'prefix'],
@@ -91,26 +90,38 @@ The %info structure is initialized as follows:
     # basic c struct
     options => ['desc', 'treat as object'],
     groups => undef,
-    creates_type => 'struct',},
+    creates_type => 'struct'},
   union => {
     # basic C union
     options => ['desc', 'treat as object'],
     groups => undef,
     creates_type => 'union'},
+  func => {
+    options => ['desc', 'no c impl'],
+    groups => undef},
+  callback => {
+    options => ['desc'],
+    groups => undef,
+    creates_type => 'callback'},
   class => {
     # C++ class
-    options => ['c impl headers'],
+    options => ['c impl headers', 'indirect', 'base'],
     groups => undef,
     creates_type => 'class'},
   errors => {}, # possible errors
   method => {
     # A class method
-    options => ['desc', 'posib err', 'c func', 'const', 'no conv', 'on conv error',
-		'c only', 'c impl', 'cxx impl', 'cc extra'],
+    options => ['desc', 'posib err', 'c func', 'const',
+		'c only', 'c impl', 'no c impl', 'cxx impl'],
     groups => undef},
   constructor => {
     # A class constructor
-    options => ['returns alt type', 'c impl', 'desc'],
+    options => ['desc', 'returns alt type', 
+                'no c impl', 'c impl', 'posib err', 'conversion'],
+    groups => 'types'},
+  'posib err constructor' => {
+    # A class constructor
+    options => ['desc', 'no c impl'],
     groups => 'types'},
   destructor => {
     # A class destructor
@@ -121,8 +132,8 @@ The %info structure is initialized as follows:
 
 =pod
 
-In addition to the categories listed above a "methods" category by
-be specified in under the class category.  A "methods" category is
+In addition to the categories listed above a "methods" catagory by
+be specified in under the class category.  A "methods" catagory is
 created for each methods group under the name "<methods name> methods"
 When groups is undefined a type name may be specified in place of
 a category
