@@ -11,10 +11,10 @@
 //   linked lists.  Nodes are allocated in large chunks instead of
 //   one at a time which means that getting a new node is very fast.
 //   Nodes returned are not initialized in any way.  A placement new
-//   must be used to construct the data member and the destructor
+//   must be used to construct the the data member and the destructor
 //   must explicitly be called.
 
-namespace acommon {
+namespace aspell {
 
   template <typename T>
   class BlockSList {
@@ -26,6 +26,10 @@ namespace acommon {
       T      data;
     };
 
+    // Delete a node, usefull when deleting objects that were
+    // constructed inplace.
+    static void del_node(Node * node) {if (node) node->data.~T();}
+    
   private:
 
     void * first_block;
@@ -52,8 +56,8 @@ namespace acommon {
     }
 
     void remove_node(Node * n) 
-      // marks the node as available
-      // Note: the node's data member destructor is NOT called
+      // marks the node as availabe
+      // Note: the node's data memeber destructor is NOT called
     {
       n->next = first_available;
       first_available = n;
