@@ -4,17 +4,24 @@
 // license along with this library if you did not you can find
 // it at http://www.gnu.org/.
 
-//#include <stdio.h>
+#include <stdio.h>
 //#define DEBUG {fprintf(stderr,"File: %s(%i)\n",__FILE__,__LINE__);}
 #include <string.h>
 #include <stdlib.h>
 #include <assert.h>
-
-#include "dirs.h"
+#ifdef WIN32PORT
+#include <windows.h>// GetModuleFilename
+#include <shlobj.h> // SHGetSpecialFolderLocation
+extern void * get_module_handle();
+#endif
 #include "settings.h"
+#include "dirs.h"
 
 #ifdef USE_LOCALE
 # include <locale.h>
+#endif
+
+#ifdef HAVE_LANGINFO_CODESET
 # include <langinfo.h>
 #endif
 
@@ -35,7 +42,9 @@
 #include "vararray.hpp"
 #include "string_list.hpp"
 
-//#include "iostream.hpp"
+#include "gettext.h"
+
+#include "iostream.hpp"
 
 #define DEFAULT_LANG "en_US"
 
@@ -62,7 +71,7 @@
 // the case of a "lset-*" this will happen in multiple config
 // "Entry"s, so care is taken to only add the arg position once.
 
-namespace acommon {
+namespace aspell {
 
   const char * const keyinfo_type_name[4] = {
     N_("string"), N_("integer"), N_("boolean"), N_("list")
@@ -74,8 +83,8 @@ namespace acommon {
   typedef Notifier * NotifierPtr;
   
   Config::Config(ParmStr name,
-		 const KeyInfo * mainbegin, 
-		 const KeyInfo * mainend)
+                 const KeyInfo * mainbegin, 
+                 const KeyInfo * mainend)
     : name_(name)
     , first_(0), insert_point_(&first_), others_(0)
     , committed_(true), attached_(false)
@@ -162,7 +171,7 @@ namespace acommon {
     for(; i != end; ++i) {
       Notifier * tmp = (*i)->clone(this);
       if (tmp != 0)
-	notifier_list.push_back(tmp);
+        notifier_list.push_back(tmp);
     }
   }
 
@@ -201,7 +210,7 @@ namespace acommon {
   }
 
   void Config::set_filter_modules(const ConfigModule * modbegin, 
-				  const ConfigModule * modend)
+                                  const ConfigModule * modend)
   {
     assert(filter_modules_ptrs.empty());
     filter_modules.clear();
@@ -209,7 +218,7 @@ namespace acommon {
   }
 
   void Config::set_extra(const KeyInfo * begin, 
-			       const KeyInfo * end) 
+                               const KeyInfo * end) 
   {
     extra_begin = begin;
     extra_end   = end;
@@ -271,7 +280,7 @@ namespace acommon {
   }
 
   bool Config::replace_notifier(const Notifier * o, 
-				      Notifier * n) 
+                                      Notifier * n) 
   {
     Vector<Notifier *>::iterator i   = notifier_list.begin();
     Vector<Notifier *>::iterator end = notifier_list.end();
@@ -356,7 +365,7 @@ namespace acommon {
 
     const Entry * cur = lookup(ki->name);
 
-    String value(cur ? cur->value : get_default(ki));
+    String value = cur ? cur->value : get_default(ki);
 
     if (value == "false") return false;
     else                  return true;
@@ -430,7 +439,7 @@ namespace acommon {
   }
 
   PosibErr<void> Config::retrieve_list(ParmStr key, 
-				       MutableContainer * m) const
+                                       MutableContainer * m) const
   {
     RET_ON_ERR_SET(keyinfo(key), const KeyInfo *, ki);
     if (ki->type != KeyInfoList) return make_err(key_not_list, ki->name);
@@ -441,26 +450,24 @@ namespace acommon {
   }
 
   static const KeyInfo * find(ParmStr key, 
-			      const KeyInfo * i, 
-			      const KeyInfo * end) 
+                              const KeyInfo * i, 
+                              const KeyInfo * end) 
   {
     while (i != end) {
       if (strcmp(key, i->name) == 0)
-	return i;
+        return i;
       ++i;
     }
     return i;
   }
 
   static const ConfigModule * find(ParmStr key, 
-				   const ConfigModule * i, 
-				   const ConfigModule * end) 
+                                   const ConfigModule * i, 
+                                   const ConfigModule * end) 
   {
-
     while (i != end) {
-      if (strcmp(key, i->name) == 0){ 
-	return i;
-      }
+      if (strcmp(key, i->name) == 0)
+        return i;
       ++i;
     }
     return i;
@@ -471,10 +478,10 @@ namespace acommon {
     typedef PosibErr<const KeyInfo *> Ret;
     {
       const KeyInfo * i;
-      i = acommon::find(key, keyinfo_begin, keyinfo_end);
+      i = aspell::find(key, keyinfo_begin, keyinfo_end);
       if (i != keyinfo_end) return Ret(i);
       
-      i = acommon::find(key, extra_begin, extra_end);
+      i = aspell::find(key, extra_begin, extra_end);
       if (i != extra_end) return Ret(i);
       
       const char * s = strncmp(key, "f-", 2) == 0 ? key + 2 : key.str();
@@ -482,28 +489,28 @@ namespace acommon {
       if (h == 0) goto err;
 
       String k(s, h - s);
-      const ConfigModule * j = acommon::find(k,
-					     filter_modules.pbegin(),
-					     filter_modules.pend());
+      const ConfigModule * j = aspell::find(k,
+                                             filter_modules.pbegin(),
+                                             filter_modules.pend());
       
       if (j == filter_modules.pend() && load_filter_hook && committed_) {
         // FIXME: This isn't quite right
         PosibErrBase pe = load_filter_hook(const_cast<Config *>(this), k);
         pe.ignore_err();
-        j = acommon::find(k,
+        j = aspell::find(k,
                           filter_modules.pbegin(),
                           filter_modules.pend());
       }
 
       if (j == filter_modules.pend()) goto err;
 
-      i = acommon::find(key, j->begin, j->end);
+      i = aspell::find(key, j->begin, j->end);
       if (i != j->end) return Ret(i);
       
       if (strncmp(key, "f-", 2) != 0) k = "f-";
       else                            k = "";
       k += key;
-      i = acommon::find(k, j->begin, j->end);
+      i = aspell::find(k, j->begin, j->end);
       if (i != j->end) return Ret(i);
     }
   err:  
@@ -560,22 +567,24 @@ namespace acommon {
 
 #if defined USE_LOCALE && defined HAVE_LANGINFO_CODESET
 
-  static inline void get_encoding(String & final_str)
+  static inline void get_encoding(const Config & c, String & final_str)
   {
     const char * codeset = nl_langinfo(CODESET);
-    if (is_ascii_enc(codeset)) codeset = "none";
+    if (ascii_encoding(c, codeset)) codeset = "none";
     final_str = codeset;
   }
 
 #else
 
-  static inline void get_encoding(String & final_str)
+  static inline void get_encoding(const Config &, String & final_str)
   {
     final_str = "none";
   }
 
 #endif
 
+  // Return the default value for the given key.  Expand the
+  // configuration string.
   String Config::get_default(const KeyInfo * ki) const
   {
     bool   in_replace = false;
@@ -591,22 +600,64 @@ namespace acommon {
         if (entry = lookup("actual-lang"), entry) {
           return entry->value;
         } else if (have("master")) {
-	  final_str = "<unknown>";
-	} else {
-	  get_lang(final_str);
-	}
-	
+          final_str = "<unknown>";
+        } else {
+          get_lang(final_str);
+        }
+        
       } else if (strcmp(i, "encoding") == 0) {
 
-        get_encoding(final_str);
+        get_encoding(*this, final_str);
 
       } else if (strcmp(i, "special") == 0) {
 
-	// do nothing
+        // do nothing
 
-      } else {
+      }
+#ifdef WIN32PORT
+      else if (strcmp(i, "prefix") == 0) {
+
+        char * buff = new char [MAX_PATH];
+        HMODULE hand = reinterpret_cast<HMODULE> (get_module_handle());
+        if (GetModuleFileName(hand, buff, MAX_PATH)) {
+          //convert all \ chars to /
+          for (char *ptr = buff; *ptr; ++ptr)
+            if ('\\' == *ptr)
+              *ptr = '/';
+          //snip off the filename leaving the path
+          char * end = strrchr(buff,'/');
+          if (end)
+            *end = 0;
+          final_str = buff;
+          delete [] buff;
+        } else {
+          printf("could not find install directory, !prefix");
+          final_str = ""; // this should not happen
+        }
+
+      } else if (strcmp(i, "home-dir") == 0) { 
+
+        //get the personal folder (e.g. "c:\My Documents")
+        char * dir = new char[MAX_PATH];
+        LPITEMIDLIST items = 0;
+        HRESULT hand = SHGetSpecialFolderLocation(0, CSIDL_PERSONAL, &items);
+        if ((NOERROR == hand) && items) {
+          if (SHGetPathFromIDList(items, dir)) {
+            for (char *ptr = dir; *ptr; ++ptr)
+              if ('\\' == *ptr)
+                *ptr = '/';
+            final_str = dir;
+          }
+          CoTaskMemFree(items);
+        }
+        delete [] dir;
+
+      }
+#endif
+      else {
       
-	abort(); // this should not happen
+        fprintf(stderr, "!%s unimplemented", i);
+        abort(); // this should not happen
       
       }
     
@@ -614,55 +665,55 @@ namespace acommon {
     
       if (!in_replace) {
 
-	if (*i == '<') {
-	  in_replace = true;
-	} else {
-	  final_str += *i;
-	}
+        if (*i == '<') {
+          in_replace = true;
+        } else {
+          final_str += *i;
+        }
 
       } else { // in_replace
       
-	if (*i == '/' || *i == ':' || *i == '|' || *i == '#' || *i == '^') {
-	  char sep = *i;
-	  String second;
-	  ++i;
-	  while (*i != '\0' && *i != '>') second += *i++;
-	  if (sep == '/') {
-	    String s1 = retrieve(replace);
-	    String s2 = retrieve(second);
-	    final_str += add_possible_dir(s1, s2);
-	  } else if (sep == ':') {
-	    String s1 = retrieve(replace);
-	    final_str += add_possible_dir(s1, second);
-	  } else if (sep == '#') {
-	    String s1 = retrieve(replace);
-	    assert(second.size() == 1);
-	    unsigned int s = 0;
-	    while (s != s1.size() && s1[s] != second[0]) ++s;
-	    final_str.append(s1, s);
-	  } else if (sep == '^') {
-	    String s1 = retrieve(replace);
-	    String s2 = retrieve(second);
-	    final_str += figure_out_dir(s1, s2);
-	  } else { // sep == '|'
-	    assert(replace[0] == '$');
-	    const char * env = getenv(replace.c_str()+1);
-	    final_str += env ? env : second;
-	  }
-	  replace = "";
-	  in_replace = false;
+        if (*i == '/' || *i == ':' || *i == '|' || *i == '#' || *i == '^') {
+          char sep = *i;
+          String second;
+          ++i;
+          while (*i != '\0' && *i != '>') second += *i++;
+          if (sep == '/') {
+            String s1 = retrieve(replace);
+            String s2 = retrieve(second);
+            final_str += add_possible_dir(s1, s2);
+          } else if (sep == ':') {
+            String s1 = retrieve(replace);
+            final_str += add_possible_dir(s1, second);
+          } else if (sep == '#') {
+            String s1 = retrieve(replace);
+            assert(second.size() == 1);
+            unsigned int s = 0;
+            while (s != s1.size() && s1[s] != second[0]) ++s;
+            final_str.append(s1, s);
+          } else if (sep == '^') {
+            String s1 = retrieve(replace);
+            String s2 = retrieve(second);
+            final_str += figure_out_dir(s1, s2);
+          } else { // sep == '|'
+            assert(replace[0] == '$');
+            const char * env = getenv(replace.c_str()+1);
+            final_str += env ? env : second.c_str();
+          }
+          replace = "";
+          in_replace = false;
 
-	} else if (*i == '>') {
+        } else if (*i == '>') {
 
-	  final_str += retrieve(replace).data;
-	  replace = "";
-	  in_replace = false;
+          final_str += retrieve(replace).data;
+          replace = "";
+          in_replace = false;
 
-	} else {
+        } else {
 
-	  replace += *i;
+          replace += *i;
 
-	}
+        }
 
       }
       
@@ -670,6 +721,7 @@ namespace acommon {
     return final_str;
   }
 
+
   PosibErr<String> Config::get_default(ParmStr key) const
   {
     RET_ON_ERR_SET(keyinfo(key), const KeyInfo *, ki);
@@ -808,6 +860,7 @@ namespace acommon {
   PosibErr<void> Config::set(Entry * entry0, bool do_unescape)
   {
     StackPtr<Entry> entry(entry0);
+
     if (entry->action == NoOp)
       entry->key = base_name(entry->key.str(), &entry->action);
 
@@ -845,8 +898,7 @@ namespace acommon {
       default:
         ;
       }
-      entry->value.ensure_null_end();
-      if (do_unescape) unescape(entry->value.data());
+      if (do_unescape) unescape(entry->value.mstr());
 
       entry->next = *insert_point_;
       *insert_point_ = entry;
@@ -929,6 +981,14 @@ namespace acommon {
 
       entry->key = ki->name;
       
+      // FIXME: This is the correct thing to do but it causes problems
+      //        with changing a filter mode in "pipe" mode and probably
+      //        elsewhere.
+      //if (attached_ && !(ki->flags & KEYINFO_MAY_CHANGE)) {
+      //  pe = make_err(cant_change_value, entry->key);
+      //  goto error;
+      //}
+
       int place_holder = entry->place_holder;
       
       if (conv && ki->flags & KEYINFO_UTF8)
@@ -1050,34 +1110,34 @@ namespace acommon {
 
     const KeyInfo * next() {
       if (i == cd->keyinfo_end) {
-	if (include_extra)
-	  i = cd->extra_begin;
-	else
-	  i = cd->extra_end;
+        if (include_extra)
+          i = cd->extra_begin;
+        else
+          i = cd->extra_end;
       }
       
       module_changed = false;
       if (i == cd->extra_end) {
-	m = cd->filter_modules.pbegin();
-	if (!include_modules || m == cd->filter_modules.pend()) return 0;
-	else {
+        m = cd->filter_modules.pbegin();
+        if (!include_modules || m == cd->filter_modules.pend()) return 0;
+        else {
           i = m->begin;
           module_changed = true;
         }
       }
 
       if (m == 0){
-	return i++;
+        return i++;
       }
 
       if (m == cd->filter_modules.pend()){
-	return 0;
+        return 0;
       }
 
       while (i == m->end) {
-	++m;
-	if (m == cd->filter_modules.pend()) return 0;
-	else {
+        ++m;
+        if (m == cd->filter_modules.pend()) return 0;
+        else {
           i = m->begin;
           module_changed = true;
         }
@@ -1160,7 +1220,7 @@ namespace acommon {
   }
 
   void Config::write_to_stream(OStream & out, 
-			       bool include_extra) 
+                               bool include_extra) 
   {
     KeyInfoEnumeration * els = possible_elements(include_extra);
     const KeyInfo * i;
@@ -1191,7 +1251,7 @@ namespace acommon {
       obuf.printf("# %s (%s)\n#   %s\n",
                   i->name, _(keyinfo_type_name[i->type]), _(i->desc));
       if (i->def != 0) {
-	if (i->type != KeyInfoList) {
+        if (i->type != KeyInfoList) {
           buf.resize(strlen(i->def) * 2 + 1);
           escape(buf.data(), i->def);
           obuf.printf("# default: %s", buf.data());
@@ -1203,18 +1263,18 @@ namespace acommon {
           }
           obuf << '\n';
           const Entry * entry = lookup(i->name);
-	  if (entry) {
+          if (entry) {
             have_value = true;
             buf.resize(entry->value.size() * 2 + 1);
             escape(buf.data(), entry->value.str());
-	    obuf.printf("%s %s\n", i->name, buf.data());
+            obuf.printf("%s %s\n", i->name, buf.data());
           }
-	} else {
+        } else {
           unsigned s = obuf.size();
           ListDump ld(obuf, i->name);
           lookup_list(i, ld, false);
           have_value = s != obuf.size();
-	}
+        }
       }
       obuf << '\n';
       if (!(i->flags & KEYINFO_HIDDEN) || have_value)
@@ -1326,8 +1386,11 @@ namespace acommon {
     return no_err;
   }
 
-
-#ifdef ENABLE_WIN32_RELOCATABLE
+#if defined(WIN32_USE_PERSONAL_DIR)
+#  define HOME_DIR "!home-dir"
+#  define PERSONAL "<lang>.pws"
+#  define REPL     "<lang>.prepl"
+#elif defined(ENABLE_WIN32_RELOCATABLE)
 #  define HOME_DIR "<prefix>"
 #  define PERSONAL "<lang>.pws"
 #  define REPL     "<lang>.prepl"
@@ -1357,8 +1420,10 @@ namespace acommon {
        N_("location of the main word list")}
     , {"encoding",   KeyInfoString, "!encoding",
        N_("encoding to expect data to be in"), KEYINFO_COMMON}
+    //, {"encoding-layers",   KeyInfoString, "!encoding",
+    //   N_("encoding to expect data to be in"), KEYINFO_COMMON}
     , {"filter",   KeyInfoList  , "url",
-       N_("add or removes a filter"), KEYINFO_MAY_CHANGE | KEYINFO_COMMON}
+       N_("add or removes a filter"), KEYINFO_MAY_CHANGE}
     , {"filter-path", KeyInfoList, DICT_DIR,
        N_("path(s) aspell looks for filters")}
     //, {"option-path", KeyInfoList, DATA_DIR,
@@ -1375,8 +1440,7 @@ namespace acommon {
        /* TRANSLATORS: It is OK if this is longer than 50 chars */
        N_("ignore accents when checking words -- CURRENTLY IGNORED"), KEYINFO_MAY_CHANGE | KEYINFO_HIDDEN}
     , {"ignore-case", KeyInfoBool  , "false",
-       /* TRANSLATORS: It is OK if this is longer than 50 chars */
-       N_("ignore case when checking words -- CURRENTLY IGNORED"), KEYINFO_MAY_CHANGE | KEYINFO_HIDDEN}
+       N_("ignore case when checking words"), KEYINFO_MAY_CHANGE}
     , {"ignore-repl", KeyInfoBool  , "false",
        N_("ignore commands to store replacement pairs"), KEYINFO_MAY_CHANGE}
     , {"jargon",     KeyInfoString, "",
@@ -1413,6 +1477,10 @@ namespace acommon {
     , {"personal", KeyInfoString, PERSONAL,
        N_("personal dictionary file name")}
     , {"personal-path", KeyInfoString, "<home-dir/personal>", 0}
+    , {"personal-no-hint", KeyInfoBool, "false",
+       N_("don't write size hint to personal dictionary when saving")}
+    , {"personal-sort", KeyInfoBool, "false",
+       N_("sort personal dictionary when saving")}
     , {"prefix",   KeyInfoString, PREFIX,
        N_("prefix directory")}
     , {"repl",     KeyInfoString, REPL,
@@ -1425,7 +1493,7 @@ namespace acommon {
     , {"run-together-min",    KeyInfoInt,   "3",
        N_("minimal length of interior words"), KEYINFO_MAY_CHANGE}
     , {"save-repl", KeyInfoBool  , "true",
-       N_("save replacement pairs on save all"), KEYINFO_MAY_CHANGE}
+       N_("save replacement pairs on save all")}
     , {"set-prefix", KeyInfoBool, "true",
        N_("set the prefix based on executable location")}
     , {"size",          KeyInfoString, "+60",
@@ -1452,7 +1520,12 @@ namespace acommon {
        N_("search path for word list information files"), KEYINFO_HIDDEN}
     , {"warn", KeyInfoBool, "true",
        N_("enable warnings")}
-    
+#ifdef WIN32PORT
+    , {"dict-subdir", KeyInfoString, "dicts",
+       N_("sub directory for dictionaries")}
+    , {"data-subdir", KeyInfoString, "data",
+       N_("sub directory for other data")}
+#endif
     
     //
     // These options are generally used when creating dictionaries
@@ -1497,6 +1570,7 @@ namespace acommon {
        N_("suggest possible replacements"), KEYINFO_MAY_CHANGE}
     , {"time"   , KeyInfoBool, "false",
        N_("time load time and suggest time in pipe mode"), KEYINFO_MAY_CHANGE}
+
     };
 
   const KeyInfo * config_impl_keys_begin = config_keys;
@@ -1506,8 +1580,8 @@ namespace acommon {
   Config * new_basic_config() { 
     aspell_gettext_init();
     return new Config("aspell",
-		      config_impl_keys_begin,
-		      config_impl_keys_end);
+                      config_impl_keys_begin,
+                      config_impl_keys_end);
   }
   
 }
