@@ -13,12 +13,13 @@
 #include "string.hpp"
 #include "vector.hpp"
 
-namespace acommon {
+namespace aspell {
 
   class OStream;
   class KeyInfoEnumeration;
   class StringPairEnumeration;
   class MutableContainer;
+  class Cacheable;
   struct Conv;
 
   // The Config class is used to hold configuration information.
@@ -160,15 +161,15 @@ namespace acommon {
 
     String temp_str;
 
-    PosibErr<ConfigModule *> (* load_filter_hook)(Config * config, ParmStr value);
+    PosibErr<const ConfigModule *> (* load_filter_hook)(Config * config, ParmStr value);
     Notifier * filter_mode_notifier;
 
     Vector<ConfigModule>      filter_modules;
-    Vector<class Cacheable *> filter_modules_ptrs;
+    Vector<Cacheable *> filter_modules_ptrs;
 
     Config(ParmStr name,
-	   const KeyInfo * mainbegin, 
-	   const KeyInfo * mainend);
+           const KeyInfo * mainbegin, 
+           const KeyInfo * mainend);
 
     Config(const Config &);
     ~Config();
@@ -231,7 +232,7 @@ namespace acommon {
 
   Config * new_config();
   Config * new_basic_config(); // config which doesn't require any
-			       // external symbols
+                               // external symbols
 
   class NotifierEnumeration {
     // no copy and destructor needed
@@ -243,7 +244,7 @@ namespace acommon {
     const Notifier * next() {
       const Notifier * temp = *i;
       if (i != end)
-	++i;
+        ++i;
       return temp;
     }
     bool at_end() const {return i == end;}
