@@ -16,7 +16,7 @@
 #include "filter.hpp"
 #include "cache.hpp"
 
-namespace acommon {
+namespace aspell {
 
   class OStream;
   class Config;
@@ -49,9 +49,9 @@ namespace acommon {
                         CharVector & out) const = 0;
     virtual PosibErr<void> encode_ec(const FilterChar * in, const FilterChar * stop, 
                                      CharVector & out, ParmStr orig) const = 0;
-    // may convert inplace
-    virtual bool encode(FilterChar * & in, FilterChar * & stop, 
-                        FilterCharVector & buf) const {return false;}
+    virtual void encode(const FilterChar * in, const FilterChar * stop, 
+                        FilterCharVector & buf) const = 0;
+
     static PosibErr<Encode *> get_new(const String &, const Config *);
     virtual ~Encode() {}
   };
@@ -83,7 +83,7 @@ namespace acommon {
       String name;
       NormTable<ToUniNormEntry> * data;
       NormTable<ToUniNormEntry> * ptr;
-      ToUniTable() : data(), ptr() {}
+      ToUniTable() : data(0), ptr(0) {}
     };
     typedef Vector<ToUniTable> ToUni;
     Vector<ToUniTable> to_uni;
@@ -91,20 +91,20 @@ namespace acommon {
   };
 
   typedef FilterCharVector ConvertBuffer;
+  struct Encoding;
 
   class Convert {
-  private:
-    CachePtr<Decode> decode_d;
+  protected:
+    String in_code_;
+    CachePtr<Decode> decode_c;
+    StackPtr<Decode> decode_s;
     Decode * decode_;
-    CachePtr<Encode> encode_d;
+    String out_code_;
+    CachePtr<Encode> encode_c;
+    StackPtr<Encode> encode_s;
     Encode * encode_;
     CachePtr<NormTables> norm_tables_;
-    DirectConv * conv_;
-
-    static const size_t memory_size = 96;
-    char memory[memory_size];
-
-    ConvertBuffer buf_;
+    StackPtr<DirectConv> conv_;
 
     static const unsigned int null_len_ = 4; // POSIB FIXME: Be more precise
 
@@ -113,18 +113,23 @@ namespace acommon {
 
   public:
     Convert() {}
+    virtual ~Convert();
 
-    // This filter is used when the convert method is called.  It must
-    // be set up by an external entity as this class does not set up
-    // this class in any way.
-    Filter filter;
+    enum InitError {NoError = 0, UnknownDecoder, UnknownEncoder, OtherError};
 
-    PosibErr<void> init(const Config &, ParmStr in, ParmStr out);
-    PosibErr<void> init_norm_to(const Config &, ParmStr in, ParmStr out);
-    PosibErr<void> init_norm_from(const Config &, ParmStr in, ParmStr out);
-    
-    const char * in_code() const   {return decode_->key.c_str();}
-    const char * out_code() const  {return encode_->key.c_str();}
+    struct InitRet {
+      InitError error;
+      PosibErr<void> error_obj;
+      InitRet() : error(NoError) {}
+    };
+
+    InitRet init(const Config &, ParmStr in, ParmStr out);
+    InitRet init_norm_to(const Config &, ParmStr in, ParmStr out, 
+                         ParmStr norm_form);
+    InitRet init_norm_from(const Config &, ParmStr in, ParmStr out);
+
+    const char * in_code()  const {return in_code_.str();}
+    const char * out_code() const {return out_code_.str();}
 
     void append_null(CharVector & out) const
     {
@@ -134,7 +139,7 @@ namespace acommon {
 
     unsigned int null_len() const {return null_len_;}
   
-    // this filters will generally not translate null characters
+    // these filters will generally not translate null characters
     // if you need a null character at the end, add it yourself
     // with append_null
 
@@ -145,13 +150,21 @@ namespace acommon {
 		CharVector & out) const
       {encode_->encode(in,stop,out);}
 
-    bool encode(FilterChar * & in, FilterChar * & stop, 
+    void encode(const FilterChar * in, const FilterChar * stop, 
                 FilterCharVector & buf) const
-      {return encode_->encode(in,stop,buf);}
+      {encode_->encode(in,stop,buf);}
+
+    // convert has the potential to use internal buffers and
+    // is therefore not const.  It is also not thread safe
+    // and I have no intention to make it thus.
+
+    virtual void convert(const char * in, int size, CharVector & out) = 0;
+
+  protected:
 
     // does NOT pass it through filters
     // DOES NOT use an internal state
-    void convert(const char * in, int size, CharVector & out, ConvertBuffer & buf) const
+    void simple_convert(const char * in, int size, CharVector & out, ConvertBuffer & buf) const
     {
       if (conv_) {
 	conv_->convert(in,size,out);
@@ -164,89 +177,178 @@ namespace acommon {
 
     // does NOT pass it through filters
     // DOES NOT use an internal state
-    PosibErr<void> convert_ec(const char * in, int size, CharVector & out, 
-                              ConvertBuffer & buf, ParmStr orig) const
+    PosibErr<void> simple_convert_ec(const char * in, int size, CharVector & out, 
+                                     ConvertBuffer & buf, ParmStr orig) const
     {
       if (conv_) {
 	RET_ON_ERR(conv_->convert_ec(in,size,out, orig));
       } else {
         buf.clear();
         RET_ON_ERR(decode_->decode_ec(in, size, buf, orig));
-       RET_ON_ERR(encode_->encode_ec(buf.pbegin(), buf.pend(), 
+        RET_ON_ERR(encode_->encode_ec(buf.pbegin(), buf.pend(), 
                                       out, orig));
       }
       return no_err;
     }
+  };
 
+  class SimpleConvert : public Convert
+  {
+  public:
+    // does NOT pass it through filters
+    // DOES NOT use an internal state
+    void convert(const char * in, int size, CharVector & out, ConvertBuffer & buf) const
+    {
+      simple_convert(in, size, out, buf);
+    }
 
+    // does NOT pass it through filters
+    // DOES NOT use an internal state
+    PosibErr<void> convert_ec(const char * in, int size, CharVector & out, 
+                              ConvertBuffer & buf, ParmStr orig) const
+    {
+      return simple_convert_ec(in, size, out, buf, orig);
+    }
+
+    void convert(const char * in, int size, CharVector & out) 
+    {
+      simple_convert(in, size, out, buf_);
+    }
+  private:
+    ConvertBuffer buf_;    
+  };
+
+  class FullConvert : public Convert
+  {
+  private:
+
+    ConvertBuffer buf_;
+
+    // This filter is used when the convert method is called.  It must
+    // be set up by an external entity as this class does not set up
+    // this class in any way.
+    Filter filter_;
+
+    void add_filter_codes();
+
+  public:
+    
+    PosibErr<void> add_filters(Config * c, 
+                               bool use_encoder, 
+                               bool use_filter, 
+                               bool use_decoder) 
+    {
+      RET_ON_ERR(setup_filter(filter_, c, use_encoder, use_filter, use_decoder));
+      add_filter_codes();
+      return no_err;
+    }
+    void add_filter(IndividualFilter * f) {
+      filter_.add_filter(f);
+      add_filter_codes();
+    }
+
+    Filter * shallow_copy_filter() {return filter_.shallow_copy();}
+      
     // convert has the potential to use internal buffers and
     // is therefore not const.  It is also not thread safe
     // and I have no intention to make it thus.
 
     void convert(const char * in, int size, CharVector & out) {
-      if (filter.empty()) {
-        convert(in,size,out,buf_);
+      if (filter_.empty()) {
+        simple_convert(in,size,out,buf_);
       } else {
         generic_convert(in,size,out);
       }
     }
 
     void generic_convert(const char * in, int size, CharVector & out);
-    
+
+    void filter(FilterChar * & start, FilterChar * & stop)
+      {if (!filter_.empty()) filter_.process(start, stop);}
+
   };
 
-  bool operator== (const Convert & rhs, const Convert & lhs);
+  //bool operator== (const Convert & rhs, const Convert & lhs);
 
+  // Enc MAY NOT be part of buf
   const char * fix_encoding_str(ParmStr enc, String & buf);
 
-  bool is_ascii_enc(ParmStr enc);
+  // Follows any alias files
+  // currently only follows a single level
+  // it is ok if enc and buf are part of the same string.
+  const char * resolve_alias(const Config & c, ParmStr enc, String & buf);
+
+  void get_base_enc(String & res, ParmStr enc);
+
+  // also returns true if the encoding is unknown
+  bool ascii_encoding(const Config & c, ParmStr enc0);
 
   enum Normalize {NormNone, NormFrom, NormTo};
 
   PosibErr<Convert *> internal_new_convert(const Config & c, 
                                            ParmString in, ParmString out,
                                            bool if_needed,
-                                           Normalize n);
+                                           Normalize n,
+                                           bool simple,
+                                           Convert::InitRet = Convert::InitRet());
   
-  static inline PosibErr<Convert *> new_convert(const Config & c,
-                                                ParmStr in, ParmStr out,
-                                                Normalize n)
+  static inline PosibErr<SimpleConvert *> 
+  new_simple_convert(const Config & c, ParmStr in, ParmStr out, Normalize n)
   {
-    return internal_new_convert(c,in,out,false,n);
+    PosibErr<Convert *> pe = internal_new_convert(c,in,out,false,n,true);
+    if (pe.has_err()) return (PosibErrBase &)pe;
+    return (SimpleConvert *)pe.data;
   }
   
-  static inline PosibErr<Convert *> new_convert_if_needed(const Config & c,
-                                                          ParmStr in, ParmStr out,
-                                                          Normalize n)
+  static inline PosibErr<SimpleConvert *> 
+  new_simple_convert_if_needed(const Config & c, ParmStr in, ParmStr out, Normalize n)
   {
-    return internal_new_convert(c,in,out,true,n);
+    PosibErr<Convert *> pe = internal_new_convert(c,in,out,true,n,true);
+    if (pe.has_err()) return (PosibErrBase &)pe;
+    return (SimpleConvert *)pe.data;
+  }
+
+  static inline PosibErr<FullConvert *> 
+  new_full_convert(const Config & c, ParmStr in, ParmStr out, Normalize n)
+  {
+    PosibErr<Convert *> pe = internal_new_convert(c,in,out,false,n,false);
+    if (pe.has_err()) return (PosibErrBase &)pe;
+    return (FullConvert *)pe.data;
+  }
+  
+  static inline PosibErr<FullConvert *> 
+  new_full_convert_if_needed(const Config & c, ParmStr in, ParmStr out, Normalize n)
+  {
+    PosibErr<Convert *> pe = internal_new_convert(c,in,out,true,n,false);
+    if (pe.has_err()) return (PosibErrBase &)pe;
+    return (FullConvert *)pe.data;
   }
 
   struct ConvObj {
-    Convert * ptr;
-    ConvObj(Convert * c = 0) : ptr(c) {}
+    SimpleConvert * ptr;
+    ConvObj(SimpleConvert * c = 0) : ptr(c) {}
     ~ConvObj() {delete ptr;}
     PosibErr<void> setup(const Config & c, ParmStr from, ParmStr to, Normalize norm)
     {
       delete ptr;
       ptr = 0;
-      PosibErr<Convert *> pe = new_convert_if_needed(c, from, to, norm);
+      PosibErr<SimpleConvert *> pe = new_simple_convert_if_needed(c, from, to, norm);
       if (pe.has_err()) return pe;
       ptr = pe.data;
       return no_err;
     }
-    operator const Convert * () const {return ptr;}
+    operator const SimpleConvert * () const {return ptr;}
   private:
     ConvObj(const ConvObj &);
     void operator=(const ConvObj &);
   };
 
   struct ConvP {
-    const Convert * conv;
+    const SimpleConvert * conv;
     ConvertBuffer buf0;
     CharVector buf;
     operator bool() const {return conv;}
-    ConvP(const Convert * c = 0) : conv(c) {}
+    ConvP(const SimpleConvert * c = 0) : conv(c) {}
     ConvP(const ConvObj & c) : conv(c.ptr) {}
     ConvP(const ConvP & c) : conv(c.conv) {}
     void operator=(const ConvP & c) { conv = c.conv; }
@@ -255,7 +357,7 @@ namespace acommon {
     {
       delete conv;
       conv = 0;
-      PosibErr<Convert *> pe = new_convert_if_needed(c, from, to, norm);
+      PosibErr<SimpleConvert *> pe = new_simple_convert_if_needed(c, from, to, norm);
       if (pe.has_err()) return pe;
       conv = pe.data;
       return no_err;
@@ -265,8 +367,7 @@ namespace acommon {
       if (conv) {
         buf.clear();
         conv->convert(str, sz, buf, buf0);
-        buf.ensure_null_end();
-        return buf.data();
+        return buf.mstr();
       } else {
         return str;
       }
@@ -290,8 +391,7 @@ namespace acommon {
       if (conv) {
         buf.clear();
         conv->convert(str, -1, buf, buf0);
-        buf.ensure_null_end();
-        return buf.data();
+        return buf.mstr();
       } else {
         return str;
       }
@@ -301,13 +401,12 @@ namespace acommon {
       if (conv) {
         buf.clear();
         conv->convert(str, -1, buf, buf0);
-        buf.ensure_null_end();
-        return buf.data();
+        return buf.mstr();
       } else {
         return str;
       }
     }
-    const char * operator() (char c)
+    char * operator() (char c)
     {
       buf.clear();
       if (conv) {
@@ -316,15 +415,14 @@ namespace acommon {
       } else {
         buf.append(c);
       }
-      buf.ensure_null_end();
-      return buf.data();
+      return buf.mstr();
     }
   };
 
   struct Conv : public ConvP
   {
     ConvObj conv_obj;
-    Conv(Convert * c = 0) : ConvP(c), conv_obj(c) {}
+    Conv(SimpleConvert * c = 0) : ConvP(c), conv_obj(c) {}
     PosibErr<void> setup(const Config & c, ParmStr from, ParmStr to, Normalize norm)
     {
       RET_ON_ERR(conv_obj.setup(c,from,to,norm));
@@ -334,11 +432,11 @@ namespace acommon {
   };
 
   struct ConvECP {
-    const Convert * conv;
+    const SimpleConvert * conv;
     ConvertBuffer buf0;
     CharVector buf;
     operator bool() const {return conv;}
-    ConvECP(const Convert * c = 0) : conv(c) {}
+    ConvECP(const SimpleConvert * c = 0) : conv(c) {}
     ConvECP(const ConvObj & c) : conv(c.ptr) {}
     ConvECP(const ConvECP & c) : conv(c.conv) {}
     void operator=(const ConvECP & c) { conv = c.conv; }
@@ -346,7 +444,7 @@ namespace acommon {
     {
       delete conv;
       conv = 0;
-      PosibErr<Convert *> pe = new_convert_if_needed(c, from, to, norm);
+      PosibErr<SimpleConvert *> pe = new_simple_convert_if_needed(c, from, to, norm);
       if (pe.has_err()) return pe;
       conv = pe.data;
       return no_err;
@@ -356,8 +454,7 @@ namespace acommon {
       if (conv) {
         buf.clear();
         RET_ON_ERR(conv->convert_ec(str, sz, buf, buf0, str));
-        buf.ensure_null_end();
-        return buf.data();
+        return buf.mstr();
       } else {
         return str;
       }
@@ -371,8 +468,7 @@ namespace acommon {
       if (conv) {
         buf.clear();
         RET_ON_ERR(conv->convert_ec(str, -1, buf, buf0, str));
-        buf.ensure_null_end();
-        return buf.data();
+        return buf.mstr();
       } else {
         return str;
       }
@@ -383,8 +479,7 @@ namespace acommon {
       if (conv) {
         buf.clear();
         RET_ON_ERR(conv->convert_ec(str, -1, buf, buf0, str));
-        buf.ensure_null_end();
-        return buf.data();
+        return buf.mstr();
       } else {
         return str.str();
       }
@@ -399,7 +494,7 @@ namespace acommon {
   struct ConvEC : public ConvECP
   {
     ConvObj conv_obj;
-    ConvEC(Convert * c = 0) : ConvECP(c), conv_obj(c) {}
+    ConvEC(SimpleConvert * c = 0) : ConvECP(c), conv_obj(c) {}
     PosibErr<void> setup(const Config & c, ParmStr from, ParmStr to, Normalize norm)
     {
       RET_ON_ERR(conv_obj.setup(c,from,to,norm));
@@ -408,6 +503,74 @@ namespace acommon {
     }
   };
 
+  // DOES NOT take ownership of the conversion filter
+  struct FullConv {
+    FullConvert * conv;
+    CharVector buf;
+    operator bool() const {return conv;}
+    FullConv(FullConvert * c = 0) : conv(c) {}
+  private:
+    FullConv(const FullConv & c) : conv(c.conv) {}
+    void operator=(const FullConv & c) { conv = c.conv; }
+  public:
+    void reset(FullConvert * c = 0) {conv = c;}
+    char * operator() (char * str, size_t sz)
+    {
+      if (conv) {
+        buf.clear();
+        conv->convert(str, sz, buf);
+        return buf.mstr();
+      } else {
+        return str;
+      }
+    }
+    const char * operator() (const char * str, size_t sz)
+    {
+      if (conv) {
+        buf.clear();
+        conv->convert(str, sz, buf);
+        return buf.str();
+      } else {
+        return str;
+      }
+    }
+    char * operator() (MutableString str)
+    {
+      return operator()(str.str, str.size);
+    }
+    char * operator() (char * str)
+    {
+      if (conv) {
+        buf.clear();
+        conv->convert(str, -1, buf);
+        return buf.mstr();
+      } else {
+        return str;
+      }
+    }
+    const char * operator() (ParmStr str)
+    {
+      if (conv) {
+        buf.clear();
+        conv->convert(str, -1, buf);
+        return buf.mstr();
+      } else {
+        return str;
+      }
+    }
+    char * operator() (char c)
+    {
+      buf.clear();
+      if (conv) {
+        char str[2] = {c, 0};
+        conv->convert(str, 1, buf);
+      } else {
+        buf.append(c);
+      }
+      return buf.mstr();
+    }
+  };
+
   struct MBLen 
   {
     enum Encoding {Other, UTF8, UCS2, UCS4} encoding;
