@@ -17,7 +17,8 @@
 // NOTE: See iostream.hpp for the standard stream (ie standard input,
 //       output, error)
 
-namespace acommon {
+namespace aspell {
+
   class String;
 
   class FStream : public IStream, public OStream
@@ -33,7 +34,7 @@ namespace acommon {
       : IStream('\n'), file_(f), own_(own) {}
     ~FStream() {close();}
 
-    PosibErr<void> open(ParmString, const char *);
+    PosibErr<void> open(ParmStr, const char *);
     void close();
  
     operator bool() {return file_ != 0 && !feof(file_) && !ferror(file_);}
@@ -62,7 +63,7 @@ namespace acommon {
 
     // These perform raw io with any sort of formating
     bool read(void *, unsigned int i);
-    void write(ParmString);
+    void write(ParmStr);
     void write(char c);
     void write(const void *, unsigned int i);
 
@@ -90,7 +91,9 @@ namespace acommon {
     FStream & operator>> (String &);
     FStream & operator>> (unsigned int &);
     FStream & operator>> (int &);
-    FStream & operator<< (ParmString);
+    FStream & operator<< (const char *);
+    FStream & operator<< (const String &);
+    FStream & operator<< (ParmStr);
     FStream & operator<< (unsigned int);
     FStream & operator<< (int);
     FStream & operator<< (double);
