@@ -16,14 +16,18 @@
 #define DISTRIBNET_LOCK__HPP
 
 #include "settings.h"
-#include <pthread.h>
 
-namespace acommon {
-
-#ifndef __GNUC__
-#define __attribute__(x)
+#ifdef USE_POSIX_MUTEX
+#  include <pthread.h>
+#elif defined (WIN32PORT)
+# ifdef _DEBUG
+#  include <stdio.h> //printf
+# endif
+# include "minwin.h" //minimum windows declarations.
 #endif
 
+namespace aspell {
+
 #define LOCK(l) const Lock the_lock(l);
 
 #ifdef USE_POSIX_MUTEX
@@ -38,6 +42,29 @@ namespace acommon {
     void lock() {pthread_mutex_lock(&l_);}
     void unlock() {pthread_mutex_unlock(&l_);}
   };
+#elif defined(WIN32PORT)
+  class Mutex {
+    HANDLE hMutex;
+  private:
+    Mutex(const Mutex &);
+    void operator=(const Mutex &);
+  public:
+    Mutex() {hMutex = CreateMutex(NULL,FALSE,NULL);}
+    ~Mutex() {CloseHandle(hMutex);}
+    void lock() {
+      long rc = WaitForSingleObject(hMutex,INFINITE);
+      #ifdef _DEBUG
+      if (rc == WAIT_ABANDONED) {
+        DWORD err = GetLastError();
+        char buff[131];
+        FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM,0,err,0,buff,sizeof(buff),0);
+        printf(buff);
+      }
+      #endif
+    }
+    void unlock() {ReleaseMutex(hMutex);}
+  };
+
 #else
   class Mutex {
   private:
@@ -60,6 +87,7 @@ namespace acommon {
     Lock(Mutex * l) : lock_(l) {if (lock_) lock_->lock();}
     ~Lock() {if (lock_) lock_->unlock();}
   };
-}
+};
 
 #endif
+
