@@ -1,9 +1,7 @@
 
 #include "objstack.hpp"
 
-namespace acommon {
-
-using namespace std;
+namespace aspell {
 
 void ObjStack::setup_chunk()
 {
@@ -35,7 +33,6 @@ ObjStack::~ObjStack()
 
 size_t ObjStack::calc_size()
 {
-  Node * ptr = first;
   size_t size = 0;
   for (Node * p = first; p; p = p->next)
     size += chunk_size;
@@ -59,8 +56,12 @@ void ObjStack::new_chunk()
 
 void ObjStack::reset()
 {
-  reserve = first->next;
-  first->next = 0;
+  assert(first_free->next == 0);
+  if (first->next) {
+    first_free->next = reserve;
+    reserve = first->next;
+    first->next = 0;
+  } 
   first_free = first;
   setup_chunk();
 }
