@@ -9,7 +9,7 @@
 
 #include <assert.h>
 
-namespace acommon {
+namespace aspell {
   
   template <typename T>
   class StackPtr {
@@ -22,7 +22,7 @@ namespace acommon {
     // and operator* and StackPtr(T *) will be used.  The explicit
     // doesn't protect us here due to PosibErr
     StackPtr(const StackPtr & other);
-    // because I am paranoid
+    // becuase I am paranoid
     StackPtr & operator=(const StackPtr & other);
 
   public:
