@@ -18,7 +18,20 @@
 #include "ostream.hpp"
 #include "istream.hpp"
 
-namespace acommon {
+//
+// aspell::String is similar to std::string, but without many of the
+// extra non-stl like methods.  The string is guaranteed to be stored
+// in a continues areas of memory but is not guaranteed to be null
+// terminated.  However, space is always allocated for the null
+// characters.  Thus, the c_str() method will never invalided any
+// exiting pointers.  The string is also null terminated when accesed
+// via the str() and mstr() methods.  In addition the method
+// ensure_null_end() will null terminate the string.  Once null
+// terminated the string will remain as such until the length of the
+// string changes.
+//
+
+namespace aspell {
 
   template <typename Ret> class PosibErr;
   
@@ -34,13 +47,30 @@ namespace acommon {
     char * begin_;
     char * end_;
     char * storage_end_;
-    void assign_only(const char * b, size_t size) 
+
+    void assign_only_nonnull(const char * b, unsigned size)
     {
       begin_ = (char *)malloc(size + 1);
       memmove(begin_, b, size);
       end_   = begin_ + size;
       storage_end_ = end_ + 1;
     }
+    void zero() 
+    {
+      begin_ = 0;
+      end_ = 0;
+      storage_end_ = 0;
+    }
+    void assign_only(const char * b)
+    {
+      if (b && *b) assign_only_nonnull(b, strlen(b));
+      else zero();
+    }
+    void assign_only(const char * b, unsigned size) 
+    {
+      if (b && size > 0) assign_only_nonnull(b, size);
+      else zero();
+    }
     void reserve_i(size_t s = 0);
   public:
     void reserve(size_t s) 
@@ -111,8 +141,8 @@ namespace acommon {
     //
 
     String() : begin_(0), end_(0), storage_end_(0) {}
-    String(const char * s) {assign_only(s, strlen(s));}
-    String(const char * s, unsigned int size) {assign_only(s, size);}
+    String(const char * s) {assign_only(s);}
+    String(const char * s, unsigned size) {assign_only(s, size);}
     String(ParmStr s) {assign_only(s, s.size());}
     String(MutableString s) {assign_only(s.str, s.size);}
     String(const String & other) {assign_only(other.begin_, other.end_-other.begin_);}
@@ -132,7 +162,11 @@ namespace acommon {
     }
     void assign(const char * b) 
     {
-      assign(b, strlen(b));
+      if (b) assign(b, strlen(b));
+    }
+    void assign(const char * b, const char * e)
+    {
+      assign(b, e - b);
     }
     String & operator= (const char * s) {
       assign(s);
@@ -160,7 +194,7 @@ namespace acommon {
     String & append(const void * str, unsigned int sz)
     {
       reserve(size() + sz);
-      memcpy(end_, str, sz);
+      if (sz > 0) memcpy(end_, str, sz);
       end_ += sz;
       return *this;
     }
@@ -377,12 +411,12 @@ namespace acommon {
     }
   };
 
-  inline String operator+ (ParmStr rhs, ParmStr lhs)
+  inline String operator+ (ParmStr lhs, ParmStr rhs)
   {
     String tmp;
-    tmp.reserve(rhs.size() + lhs.size());
-    tmp += rhs;
+    tmp.reserve(lhs.size() + rhs.size());
     tmp += lhs;
+    tmp += rhs;
     return tmp;
   }
 
@@ -462,7 +496,7 @@ namespace acommon {
 
 namespace std
 {
-  template<> static inline void swap(acommon::String & x, acommon::String & y) {return x.swap(y);}
+  template<> static inline void swap(aspell::String & x, aspell::String & y) {x.swap(y);}
 }
 
 #endif
