@@ -50,6 +50,7 @@ while ($filename=shift) {
       ( $_=~s/^STATIC[ \t]+//i) && (($feature=uc $_ ) || 1) &&
        (${$filter}{$feature}="new_aspell_".$filtername."_".(lc $_)) && next;
       ( $_=~s/^ASPELL[ \t]+//i) && next;
+      ( $_=~/^LIB-FILE/i) && next;
       ( $_=~/^OPTION[ \t]+/i) || 
        (die "Invalid general key in $filename on line $.");
     }
@@ -70,7 +71,7 @@ while ($filename=shift) {
     ( $_=~s/^TYPE[ \t]+//i) &&
      (${$option}{"TYPE"}=$_) && next;
     ( $_=~s/^DEF(?:AULT)[ \t]+//i) && (($_=~s/\\(?=[ \t])//g) || 1) &&
-     (((${$option}{"DEFAULT"} ne "") && (${$option}{"DEFAULT"}.=",")) ||1) &&
+     (((${$option}{"DEFAULT"} ne "") && (${$option}{"DEFAULT"}.=":")) ||1) &&
      (${$option}{"DEFAULT"}.=prep_str($_)) && next;
     ( $_=~s/^DES(?:CRIPTION)[ \t]+//i) && (($_=~s/\\(?=[ \t])//g) || 1) &&
      (${$option}{"DESCRIPTION"}=prep_str($_)) && next;
@@ -129,37 +130,24 @@ printf STATICFILTERS "\n  };\n";
 printf STATICFILTERS "\n  const unsigned int standard_filters_size = ".
                          "sizeof(standard_filters)/sizeof(FilterEntry);\n";
 
-#create filter mode structure 
-#FIXME obsolete as mode specification is done
-#by user readable and changeable text files
-#printf STATICFILTERS "\n  static KeyInfo modes_module[] = {\n";
-#printf STATICFILTERS "    {\"fm-email\",KeyInfoList,\"url,email\",0},\n";
-#printf STATICFILTERS "    {\"fm-none\",KeyInfoList,\"\",0},\n";
-#printf STATICFILTERS "    {\"fm-sgml\",KeyInfoList,\"url,sgml\",0},\n";
-#printf STATICFILTERS "    {\"fm-tex\",KeyInfoList,\"url,tex\",0},\n";
-#printf STATICFILTERS "    {\"fm-url\",KeyInfoList,\"url\",0},\n  };\n";
-#printf STATICFILTERS "\n  const KeyInfo * modes_module_begin = modes_module;\n";
-#printf STATICFILTERS "\n  const KeyInfo * modes_module_end = modes_module+".
-#                        "sizeof(modes_module)/sizeof(KeyInfo);\n";
 
 #create KeyInfo structures for each static filter
 while ($filter = shift @filterhashes) {
-  printf STATICFILTERS "\n  static KeyInfo ".${$filter}{"NAME"}."_options[] = {\n";
 
-#FIXME remove this line as it is obsolete here filter name and description are part of 
-#ConfigModule structure
-#  printf STATICFILTERS "    {\"filter-".${$filter}{"NAME"}."\",KeyInfoDescript,0,\"".
-#                                        ${$filter}{"DESCRIPTION"}."\"}";
-#
+# Only create a KeyInfo static if there are options.
+  @options = grep !/(?:NAME|(?:DE|EN)CODER|FILTER|DESCRIPTION)/ ,keys %{$filter};
+  next unless $#options >= 0;
+
+  printf STATICFILTERS "\n  static KeyInfo ".${$filter}{"NAME"}."_options[] = {\n";
 
 #create KeyInfo structs and begin end handles
   $firstopt = 1;
-  while (($name,$option)=each %{$filter}) {
-    ($name=~/(?:NAME|(?:DE|EN)CODER|FILTER|DESCRIPTION)/) && next;
+  foreach $name (@options) {
+    $option = $filter->{$name};
     ( $firstopt != 1 ) && ( printf STATICFILTERS ",\n" );
     $firstopt = 0;
     printf STATICFILTERS "    {\n".
-                            "      \"filter-${$filter}{NAME}-$name\",\n";
+                            "      \"f-${$filter}{NAME}-$name\",\n";
     (    (lc ${$option}{"TYPE"}) eq "bool") &&
       printf STATICFILTERS  "      KeyInfoBool,\n";
     ( (lc ${$option}{"TYPE"}) eq "int") &&
@@ -191,9 +179,15 @@ while ($filter = shift @rallfilters) {
   $firstopt = 0;
   printf STATICFILTERS "    {\n".
                        "      \"$filter\",0,\n".
-                       "      \"".${${filters}{$filter}}{DESCRIPTION}."\",\n" .
-                       "      ${filter}_options_begin,${filter}_options_end\n" .
-                       "    }";
+                       "      \"".${${filters}{$filter}}{DESCRIPTION}."\",\n";
+  # Check for no options
+  @options = grep !/(?:NAME|(?:DE|EN)CODER|FILTER|DESCRIPTION)/, keys %{$filters{$filter}};
+  if ($#options >= 0) {
+    printf STATICFILTERS "      ${filter}_options_begin,${filter}_options_end\n";
+  } else {
+    printf STATICFILTERS "      0,0\n";
+  }
+  printf STATICFILTERS "    }";
 } 
 printf STATICFILTERS    "\n  };\n";
 printf STATICFILTERS "\n  const ConfigModule * filter_modules_begin = ".
@@ -206,3 +200,4 @@ printf STATICFILTERS "\n  const size_t filter_modules_size = ".
 
 close STATICFILTERS;
 
+
