@@ -1,15 +1,15 @@
 // This file is part of The New Aspell
-// Copyright (C) 2002 by Kevin Atkinson and Christoph Hintermller under the GNU LGPL
-// license version 2.0 or 2.1.  You should have received a copy of the
-// LGPL license along with this library if you did not you can find it
-// at http://www.gnu.org/.
+// Copyright (C) 2002 by Kevin Atkinson and Christoph Hintermller
+// under the GNU LGPL license version 2.0 or 2.1.  You should have
+// received a copy of the LGPL license along with this library if you
+// did not you can find it at http://www.gnu.org/.
 //
 // Expansion for loading filter libraries and collections upon startup
 // was added by Christoph Hintermller
 
 #include "settings.h"
 
-#include "cache-t.hpp"
+#include "cache.hpp"
 #include "asc_ctype.hpp"
 #include "config.hpp"
 #include "enumeration.hpp"
@@ -30,16 +30,39 @@
 #include "string_map.hpp"
 #include "strtonum.hpp"
 #include "file_util.hpp"
+
 #include <stdio.h>
 
 #ifdef HAVE_LIBDL
 #  include <dlfcn.h>
 #endif
 
-namespace acommon
+namespace aspell
 {
 #include "static_filters.src.cpp"
 
+  //////////////////////////////////////////////////////////////////////////
+  //
+  // setup static filters
+  //
+
+  PosibErr<const ConfigModule *> get_dynamic_filter(Config * config, ParmStr value);
+  extern void activate_filter_modes(Config *config);
+
+  void setup_static_filters(Config * config)
+  {
+    config->set_filter_modules(filter_modules_begin, filter_modules_end);
+    activate_filter_modes(config);
+#ifdef HAVE_LIBDL
+    config->load_filter_hook = get_dynamic_filter;
+#endif
+  }
+
+  //////////////////////////////////////////////////////////////////////////
+  //
+  // 
+  //
+
 #ifdef HAVE_LIBDL
 
   struct ConfigFilterModule : public Cacheable {
@@ -86,7 +109,6 @@ namespace acommon
   //
 
   FilterEntry * get_standard_filter(ParmStr);
-  PosibErr<const ConfigModule *> get_dynamic_filter(Config * config, ParmStr value);
 
   //////////////////////////////////////////////////////////////////////////
   //
@@ -105,7 +127,7 @@ namespace acommon
 
     StackPtr<IndividualFilter> ifilter;
 
-    filter.clear();
+    //filter.clear();
 
     while ((filter_name = els.next()) != 0) {
       //fprintf(stderr, "Loading %s ... \n", filter_name);
@@ -117,8 +139,8 @@ namespace acommon
       // The following code parts assure that all filters needed and requested
       // by user are loaded properly or be reported to be missing.
       // 
-#ifdef HAVE_LIBDL
       FilterHandle decoder_handle, filter_handle, encoder_handle;
+#ifdef HAVE_LIBDL
       FilterEntry dynamic_filter;
       if (!f) {
 
@@ -255,10 +277,10 @@ namespace acommon
       {
         if ( d.value == NULL || *d.value == '\0' )
           return make_err(confusing_version).with_file(option_file,d.line_num);
-
-        PosibErr<void> peb = check_version(d.value.str);
+#ifdef FILTER_VERSION_CONTROL 
+       PosibErr<void> peb = check_version(d.value.str);
         if (peb.has_err()) return peb.with_file(option_file,d.line_num);
-
+#endif
         continue;
       }
 	  
@@ -461,6 +483,7 @@ namespace acommon
 #endif
 
   void load_all_filters(Config * config) {
+#ifdef HAVE_LIBDL
     StringList filter_path;
     String toload;
     
@@ -479,6 +502,7 @@ namespace acommon
       
       get_dynamic_filter(config, toload);
     }
+#endif
   }
 
 
