@@ -1,32 +1,33 @@
 // Copyright 2000 by Kevin Atkinson under the terms of the LGPL
 
-#include "language.hpp"
-#include "phonetic.hpp"
-#include "phonet.hpp"
-
 #include "file_util.hpp"
 #include "file_data_util.hpp"
-#include "clone_ptr-t.hpp"
+#include "clone_ptr.hpp"
+#include "posib_err.hpp"
 
-namespace aspeller {
+#include "lang_impl.hpp"
+#include "phonetic.hpp"
+#include "phonet.hpp"
+
+namespace aspell { namespace sp {
   
   class SimpileSoundslike : public Soundslike {
   private:
-    const Language * lang;
+    const LangImpl * lang;
     char first[256];
     char rest[256];
   public:
-    SimpileSoundslike(const Language * l) : lang(l) {}
+    SimpileSoundslike(const LangImpl * l) : lang(l) {}
 
     PosibErr<void> setup(Conv &) {
       memcpy(first, lang->sl_first_, 256);
       memcpy(rest,  lang->sl_rest_, 256);
       return no_err;
     }
-    
     String soundslike_chars() const {
+      int i;
       bool chars_set[256] = {0};
-      for (int i = 0; i != 256; ++i) 
+      for ( i = 0; i != 256; ++i) 
       {
         char c = first[i];
         if (c) chars_set[static_cast<unsigned char>(c)] = true;
@@ -34,7 +35,7 @@ namespace aspeller {
         if (c) chars_set[static_cast<unsigned char>(c)] = true;
       }
       String     chars_list;
-      for (int i = 0; i != 256; ++i) 
+      for ( i = 0; i != 256; ++i) 
       {
         if (chars_set[i]) 
           chars_list += static_cast<char>(i);
@@ -72,9 +73,9 @@ namespace aspeller {
 
   class NoSoundslike : public Soundslike {
   private:
-    const Language * lang;
+    const LangImpl * lang;
   public:
-    NoSoundslike(const Language * l) : lang(l) {}
+    NoSoundslike(const LangImpl * l) : lang(l) {}
 
     PosibErr<void> setup(Conv &) {return no_err;}
     
@@ -95,14 +96,39 @@ namespace aspeller {
     }
   };
 
+  class StrippedSoundslike : public Soundslike {
+  private:
+    const LangImpl * lang;
+  public:
+    StrippedSoundslike(const LangImpl * l) : lang(l) {}
+
+    PosibErr<void> setup(Conv &) {return no_err;}
+    
+    String soundslike_chars() const {
+      return get_stripped_chars(*lang);
+    }
+
+    char * to_soundslike(char * res, const char * str, int size) const 
+    {
+      return lang->LangImpl::to_stripped(res, str);
+    }
+
+    const char * name() const {
+      return "stripped";
+    }
+    const char * version() const {
+      return "1.0";
+    }
+  };
+
   class PhonetSoundslike : public Soundslike {
 
-    const Language * lang;
+    const LangImpl * lang;
     StackPtr<PhonetParms> phonet_parms;
     
   public:
 
-    PhonetSoundslike(const Language * l) : lang(l) {}
+    PhonetSoundslike(const LangImpl * l) : lang(l) {}
 
     PosibErr<void> setup(Conv & iconv) {
       String file;
@@ -157,11 +183,13 @@ namespace aspeller {
   
   PosibErr<Soundslike *> new_soundslike(ParmString name, 
                                         Conv & iconv,
-                                        const Language * lang)
+                                        const LangImpl * lang)
   {
     Soundslike * sl;
     if (name == "simple" || name == "generic") {
       sl = new SimpileSoundslike(lang);
+    } else if (name == "stripped") {
+      sl = new StrippedSoundslike(lang);
     } else if (name == "none") {
       sl = new NoSoundslike(lang);
     } else if (name == lang->name()) {
@@ -177,5 +205,5 @@ namespace aspeller {
       return sl;
     }
   }
-}
+} }
 
