@@ -37,22 +37,25 @@ using std::pair;
 
 #include "settings.h"
 
+#include "file_util.hpp"
+#include "fstream.hpp"
 #include "block_vector.hpp"
 #include "config.hpp"
 #include "data.hpp"
 #include "data_util.hpp"
 #include "errors.hpp"
-#include "file_util.hpp"
-#include "fstream.hpp"
-#include "language.hpp"
+#include "lang_impl.hpp"
 #include "stack_ptr.hpp"
 #include "objstack.hpp"
 #include "vector.hpp"
 #include "vector_hash-t.hpp"
 #include "check_list.hpp"
 #include "lsort.hpp"
+
 #include "iostream.hpp"
 
+#include "gettext.h"
+
 typedef unsigned int   u32int;
 static const u32int u32int_max = (u32int)-1;
 typedef unsigned short u16int;
@@ -71,6 +74,8 @@ typedef unsigned char byte;
 #define MAP_FAILED (-1)
 #endif
 
+using namespace aspell;
+
 #ifdef HAVE_MMAP
 
 static inline char * mmap_open(unsigned int block_size, 
@@ -108,7 +113,7 @@ static byte HAVE_AFFIX_FLAG = 1 << 7;
 static byte HAVE_CATEGORY_FLAG = 1 << 6;
 
 static byte DUPLICATE_FLAG = 1 << 4;
-// this flag is set when there is is more than one word for a
+// this flag is set when there is more than one word for a
 // particulear "clean" word such as "jello" "Jello".  It is set on all
 // but the last word of the group.  Ie, if it is set than the next
 // word when converted to its "clean" form equals the same value.
@@ -157,7 +162,7 @@ static inline const char * get_affix(const char * d) {
 
 static inline const char * get_category(const char * d) {
   int word_size = get_word_size(d);
-  if (get_flags(d) & HAVE_AFFIX_FLAG & HAVE_CATEGORY_FLAG) 
+  if (get_flags(d) & (HAVE_AFFIX_FLAG | HAVE_CATEGORY_FLAG)) 
     return d + strlen(d + word_size +  1) + 1;
   else if (get_flags(d) & HAVE_CATEGORY_FLAG)
     return d + word_size +  1;
@@ -171,7 +176,7 @@ static inline bool duplicate_flag(const char * d) {
 
 namespace {
 
-  using namespace aspeller;
+  using namespace aspell::sp;
 
   /////////////////////////////////////////////////////////////////////
   // 
@@ -182,7 +187,7 @@ namespace {
   {
     char   sl[4];
     u32int loc;
-    Jump() : sl(), loc() {}
+    Jump() {memset(this, 0, sizeof(Jump));}
   };
   
   class ReadOnlyDict : public Dictionary
@@ -193,7 +198,7 @@ namespace {
     struct WordLookupParms {
       const char * block_begin;
       WordLookupParms() {}
-      typedef BlockVector<const u32int> Vector;
+      typedef BlockVector<const u32int> Vec;
       typedef u32int                    Value;
       typedef const char *              Key;
       static const bool is_multi = false;
@@ -209,7 +214,8 @@ namespace {
       
     char *           block;
     u32int           block_size;
-    bool             block_mmaped;
+    char *           mmaped_block;
+    u32int           mmaped_size;
     const Jump * jump1;
     const Jump * jump2;
     WordLookup       word_lookup;
@@ -235,8 +241,8 @@ namespace {
 
     ~ReadOnlyDict() {
       if (block != 0) {
-	if (block_mmaped)
-	  mmap_free(block, block_size);
+	if (mmaped_block)
+	  mmap_free(mmaped_block, mmaped_size);
 	else
 	  free(block);
       }
@@ -300,8 +306,8 @@ namespace {
   static const char * const cur_check_word = "aspell default speller rowl 1.10";
 
   struct DataHead {
-    // all sizes except the last four must to divisible by:
-    static const uint align = 16;
+    // all sizes except the last four must to divisible by "align":
+    static const unsigned int align = 16;
     char check_word[64];
     u32int endian_check; // = 12345678
     char lang_hash[16];
@@ -393,9 +399,13 @@ namespace {
     affix_compressed = data_head.affix_info;
 
     block_size = data_head.block_size;
-    block = mmap_open(block_size, f, data_head.head_size);
-    block_mmaped = block != (char *)MAP_FAILED;
-    if (!block_mmaped) {
+    int offset = data_head.head_size;
+    mmaped_block = mmap_open(block_size + offset, f, 0);
+    if( mmaped_block != (char *)MAP_FAILED) {
+      block = mmaped_block + offset;
+      mmaped_size = block_size + offset;
+    } else {
+      mmaped_block = 0;
       block = (char *)malloc(block_size);
       f.seek(data_head.head_size);
       f.read(block, block_size);
@@ -445,7 +455,7 @@ namespace {
     const char * w = (const char *)wi->intr[0];
     const SensitiveCompare * c = (const SensitiveCompare *)wi->intr[1];
     const char * orig = (const char *)wi->intr[2];
-    wi->word = w;
+    convert(w,*wi);
     wi->adv_ = 0;
     prep_next(wi, w, c, orig);
   }
@@ -477,6 +487,7 @@ namespace {
     const Jump * jump1;
     const Jump * jump2;
     const char * cur;
+    const char * prev;
     int level;
     bool invisible_soundslike;
 
@@ -491,9 +502,12 @@ namespace {
   WordEntry * ReadOnlyDict::SoundslikeElements::next(int stopped_at) {
 
     //CERR << level << ":" << stopped_at << "  :";
-    //CERR << jump1->data << ":" << jump2->sl << "\n";
+    //CERR << jump1->sl << ":" << jump2->sl << "\n";
+
+  loop:
 
     const char * tmp = cur;
+    const char * p;
 
     if (level == 1 && stopped_at < 2) {
 
@@ -522,7 +536,7 @@ namespace {
 
     } else if (level == 2) {
 
-      cur = tmp = obj->word_block + jump2->loc;
+      tmp = cur = obj->word_block + jump2->loc;
       level = 3;
 
     } else if (get_offset(cur) == 0) {
@@ -542,6 +556,27 @@ namespace {
 
     cur = get_next(cur); // this will be the NEXT item looked at
 
+    p = prev;
+    prev = tmp;
+    if (p) {
+      // PRECOND:
+      // unless stopped_at >= LARGE_NUM
+      //     strlen(p) >= stopped_at
+      //     (stopped_at >= 3) implies 
+      //         strncmp(p, tmp, 3) == 0 if !invisible_soundslike
+      //         strncmp(to_sl(p), to_sl(tmp), 3) == 0 if invisible_soundslike
+      if (stopped_at == 3) {
+        if (p[3] == tmp[3]) goto loop;
+      } else if (stopped_at == 4) {
+        if (p[3] == tmp[3] && tmp[3] &&
+            p[4] == tmp[4]) goto loop;
+      } else if (stopped_at == 5) {
+        if (p[3] == tmp[3] && tmp[3] &&
+            p[4] == tmp[4] && tmp[4] &&
+            p[5] == tmp[5]) goto loop;
+      }
+    }
+    
     data.word = tmp;
     data.word_size = get_word_size(tmp);
     if (invisible_soundslike) {
@@ -549,9 +584,11 @@ namespace {
       data.what = WordEntry::Word;
     } 
     data.intr[0] = (void *)tmp;
+    
     return &data;
 
   jquit:
+    prev = 0;
     if (!*tmp) return 0;
     data.word = tmp;
     data.word_size = !tmp[1] ? 1 : !tmp[2] ? 2 : 3;
@@ -635,13 +672,13 @@ namespace {
 
 }  
 
-namespace aspeller {
+namespace aspell { namespace sp {
 
   Dictionary * new_default_readonly_dict() {
     return new ReadOnlyDict();
   }
   
-}
+} }
 
 namespace {
 
@@ -659,9 +696,10 @@ namespace {
   //       no soundslike
   //       invisible soundslike
 
-  using namespace aspeller;
+  using namespace aspell::sp;
 
   struct WordData {
+    static const unsigned struct_size;
     WordData * next;
     char * sl;
     char * aff;
@@ -669,12 +707,15 @@ namespace {
     byte sl_size;
     byte data_size;
     byte flags;
-    char word[];
+    char word[1];
   };
 
+  const unsigned WordData::struct_size = sizeof(WordData) - 1;
+  
+
   struct SoundslikeLess {
     InsensitiveCompare icomp;
-    SoundslikeLess(const Language * l) : icomp(l) {}
+    SoundslikeLess(const LangImpl * l) : icomp(l) {}
     bool operator() (WordData * x, WordData * y) const {
       int res = strcmp(x->sl, y->sl);
       if (res != 0) return res < 0;
@@ -687,7 +728,7 @@ namespace {
   struct WordLookupParms {
     const char * block_begin;
     WordLookupParms() {}
-    typedef Vector<u32int>      Vector;
+    typedef Vector<u32int>      Vec;
     typedef u32int              Value;
     typedef const char *        Key;
     static const bool is_multi = false;
@@ -711,7 +752,7 @@ namespace {
   }
 
   PosibErr<void> create (StringEnumeration * els,
-			 const Language & lang,
+			 const LangImpl & lang,
                          Config & config) 
   {
     assert(sizeof(u16int) == 2);
@@ -722,11 +763,11 @@ namespace {
                              strcmp(lang.soundslike_name(), "simple") == 0);
 
     bool affix_compress = (lang.affix() && 
-                           config.retrieve_bool("affix-compress"));
+                           config.retrieve_bool("affix-compress").data);
 
     bool partially_expand = (affix_compress &&
                              !full_soundslike &&
-                             config.retrieve_bool("partially-expand"));
+                             config.retrieve_bool("partially-expand").data);
 
     bool invisible_soundslike = false;
     if (partially_expand)
@@ -838,7 +879,7 @@ namespace {
           const char * w = p->word.str;
           s = p->word.size;
           
-          unsigned total_size = sizeof(WordData);
+          unsigned total_size = WordData::struct_size;
           unsigned data_size = s + 1;
           unsigned aff_size = strlen((const char *)p->aff);
           if (aff_size > 0) data_size += aff_size + 1;
@@ -849,7 +890,7 @@ namespace {
           if (strcmp(sl,w) == 0) sl = w;
           if (sl != w) total_size += sl_size + 1;
 
-          if (total_size - sizeof(WordData) > 240)
+          if (total_size - WordData::struct_size > 240)
             return make_err(invalid_word, MsgConv(lang)(w),
                             _("The total word length, with soundslike data, is larger than 240 characters."));
 
@@ -899,31 +940,25 @@ namespace {
     // duplicate check
     // 
     WordData * prev = first;
-    WordData * cur = first->next;
+    WordData * cur = first ? first->next : 0;
     InsensitiveEqual ieq(&lang);
     while (cur) {
       if (strcmp(prev->word, cur->word) == 0) {
-        // merge affix info if necessary
         if (!prev->aff && cur->aff) {
+          // merge affix info into previous word
           prev->flags |= HAVE_AFFIX_FLAG;
           prev->aff = cur->aff;
           prev->data_size += strlen(prev->aff) + 1;
+          prev->next = cur->next;
         } else if (prev->aff && cur->aff) {
-          unsigned l1 = strlen(prev->aff);
-          unsigned l2 = strlen(cur->aff);
-          char * aff = (char *)buf.alloc(l1 + l2 + 1);
-          memcpy(aff, prev->aff, l1);
-          prev->aff = aff;
-          aff += l1;
-          for (const char * p = cur->aff; *p; ++p) {
-            if (memchr(prev->aff, l1, *p)) continue;
-            *aff = *p;
-            ++aff;
-          }
-          *aff = '\0';
-          prev->data_size = prev->word_size + (aff - prev->aff) + 2;
+          // don't merge affix info, store both entries
+          prev->flags |= DUPLICATE_FLAG;
+          ++num_entries;
+          prev = cur;
+        } else {
+          // ignore this word
+          prev->next = cur->next;
         }
-        prev->next = cur->next;
       } else {
         if (ieq(prev->word, cur->word)) prev->flags |= DUPLICATE_FLAG;
         else ++uniq_entries;
@@ -1138,17 +1173,17 @@ namespace {
 
 }
 
-namespace aspeller {
+namespace aspell { namespace sp {
   PosibErr<void> create_default_readonly_dict(StringEnumeration * els,
                                               Config & config)
   {
-    CachePtr<Language> lang;
-    PosibErr<Language *> res = new_language(config);
+    CachePtr<LangImpl> lang;
+    PosibErr<LangImpl *> res = new_lang_impl(config);
     if (res.has_err()) return res;
     lang.reset(res.data);
     lang->set_lang_defaults(config);
     RET_ON_ERR(create(els,*lang,config));
     return no_err;
   }
-}
+} }
 
