@@ -13,17 +13,17 @@
 #ifndef __aspeller_vector_hash_hh__
 #define __aspeller_vector_hash_hh__
 
-#include <iterator>
-#include <utility>
+//#include <iterator>
+//#include <utility>
 
 #include "settings.h"
 #undef REL_OPS_POLLUTION  // FIXME
 
-namespace aspeller {
+namespace aspell { namespace sp {
 
   //
   // This hash table is implemnted as a Open Address Hash Table
-  // which uses a Vector like object to store its data.  So
+  // which uses a vector like object to store its data.  So
   // it might even be considered an adapter
   //
   
@@ -43,19 +43,13 @@ namespace aspeller {
   //  typename Value;
   class VHTIterator 
   {
-    template <class T>
-    friend bool operator== (VHTIterator<T>, VHTIterator<T>);
-#ifndef REL_OPS_POLLUTION
-    template <class T>
-    friend bool operator!= (VHTIterator<T>, VHTIterator<T>);
-#endif
   public: //but don't use
     typedef typename Parms::TableIter       TableIter;
     typedef typename Parms::HashTable       HashTable;
     TableIter   pos;
-    HashTable * hash_table; 
+    const HashTable * hash_table;
   public:
-    typedef std::bidirectional_iterator_tag             iterator_category;
+    //typedef std::bidirectional_iterator_tag             iterator_category;
     typedef typename Parms::Value                       value_type;
     // these cause problems for SUNPRO_CC
     //typedef typename std::iterator_traits<TableIter>::difference_type difference_type;
@@ -64,8 +58,8 @@ namespace aspeller {
 
     //VHTIterator vector_iterator() const {return pos;}
   public:
-    VHTIterator(TableIter p, HashTable *ht) : pos(p), hash_table(ht) {}
-    VHTIterator(TableIter p, HashTable *ht, bool) 
+    VHTIterator(TableIter p, const HashTable *ht) : pos(p), hash_table(ht) {}
+    VHTIterator(TableIter p, const HashTable *ht, bool) 
       : pos(p), hash_table(ht) 
     {
       while (pos != hash_table->vector().end()
@@ -73,7 +67,7 @@ namespace aspeller {
 	++pos;
     }
     
-    value_type & operator * () const  {return *pos;}
+    const value_type & operator * () const  {return *pos;}
     value_type * operator -> () const {return &*pos;}
     
     bool at_end() const {return pos == hash_table->vector().end();}
@@ -134,9 +128,9 @@ namespace aspeller {
   ////////////////////////////////////////////////////////
 
   // Parms is expected to have the following methods
-  //   typename Vector
-  //   typedef Vector::value_type Value
-  //   typedef Vector::size_type  Size
+  //   typename Vec
+  //   typedef Vec::value_type Value
+  //   typedef Vec::size_type  Size
   //   typename Key
   //   bool is_multi;
   //   Size hash(Key)
@@ -147,20 +141,21 @@ namespace aspeller {
 
   template <class Parms>
   class VectorHashTable {
-    typedef typename Parms::Vector           Vector;
+    typedef typename Parms::Vec           Vec;
   public:
-    typedef typename Parms::Vector           vector_type;
-    typedef typename Vector::value_type      value_type;
-    typedef typename Vector::size_type       size_type;
-    typedef typename Vector::difference_type difference_type;
+    typedef typename Parms::Vec           vector_type;
+    typedef typename Vec::value_type      value_type;
+    typedef typename Vec::size_type       size_type;
+    typedef typename Vec::difference_type difference_type;
 
-    typedef typename Vector::pointer         pointer;
-    typedef typename Vector::reference       reference;
-    typedef typename Vector::const_reference const_reference;
+    // This typedef causes problems with VC6
+    //typedef typename Vec::pointer         pointer;
+    typedef typename Vec::reference       reference;
+    typedef typename Vec::const_reference const_reference;
 
-    typedef typename Parms::Key              key_type;
+    typedef typename Parms::Key           key_type;
   public: // but don't use
-    typedef VectorHashTable<Parms>           HashTable;
+    typedef VectorHashTable<Parms>        HashTable;
   private:
     Parms parms_;
 
@@ -171,19 +166,19 @@ namespace aspeller {
   public:
     // These public functions are very dangerous and should be used with
     // great care as the modify the internal structure of the object
-    Vector & vector()       {return vector_;}
-    const Vector & vector() const {return vector_;}
+    Vec & vector()       {return vector_;}
+    const Vec & vector() const {return vector_;}
     parms_type & parms() {return parms_;}
     void recalc_size();
     void set_size(size_type s) {size_  = s;} 
 
   private:
-    Vector      vector_;
+    Vec         vector_;
     size_type   size_;
 
   public: // but don't use
-    typedef typename Vector::iterator       vector_iterator;
-    typedef typename Vector::const_iterator const_vector_iterator;
+    typedef typename Vec::iterator       vector_iterator;
+    typedef typename Vec::const_iterator const_vector_iterator;
   
   private:
     int hash1(const key_type &d) const {
@@ -214,7 +209,7 @@ namespace aspeller {
     VectorHashTable(size_type i, const Parms & p = Parms());
 
     VectorHashTable(const Parms & p = Parms())
-      : parms_(p), vector_(7), size_(0) {nonexistent_vector();}
+      : parms_(p), vector_(19), size_(0) {nonexistent_vector();}
  
     iterator begin() {return iterator(vector_.begin(), this, 1);}
     iterator end()   {return iterator(vector_.end(), this);}
@@ -223,9 +218,22 @@ namespace aspeller {
 
     std::pair<iterator, bool> insert(const value_type &);
     bool have(const key_type &) const;
-
-    iterator find(const key_type&);
-    const_iterator find(const key_type&) const;
+    iterator find(const key_type & key) 
+    {
+      MutableFindIterator it(this, key);
+      if (!it.at_end()) 
+        return iterator(vector_.begin() + it.i, this);
+      else
+        return end();
+    }
+    const_iterator find(const key_type & key) const 
+    {
+      ConstFindIterator it(this, key);
+      if (!it.at_end()) 
+        return const_iterator(vector_.begin() + it.i, this);
+      else
+        return end();
+    }
   
     size_type erase(const key_type &key);
     void erase(const iterator &p);
@@ -248,10 +256,30 @@ namespace aspeller {
       int i;
       int hash2;
       FindIterator() {}
-      FindIterator(const HashTable * ht, const key_type & k);
+    FindIterator(const HashTable * ht, const key_type & k)
+     : vector(&ht->vector())
+     , parms(&ht->parms())
+     , key(k)
+     , i(ht->hash1(k))
+     , hash2(ht->hash2(k))
+    {
+      if (!parms->is_nonexistent((*vector)[i])
+       && !parms->equal(parms->key((*vector)[i]), key))
+        adv();
+    }
+
+
     public:
       bool at_end() const {return parms->is_nonexistent((*vector)[i]);}
-      void adv();
+    void adv()
+    {
+      do {
+        i = (i + hash2) % vector->size();
+      }
+      while (!parms->is_nonexistent((*vector)[i]) &&
+       !parms->equal(parms->key((*vector)[i]), key));
+    }
+
       FindIterator & operator ++() {adv(); return *this;}
     };
     friend class FindIterator;
@@ -286,6 +314,6 @@ namespace aspeller {
     }
     
   };
-}
+} }
 
 #endif
