@@ -6,18 +6,18 @@
 #include <assert.h>
 #include "string.hpp"
 
-namespace acommon {
+namespace aspell {
   class OStream;
   class Convert;
 }
 
-namespace aspeller {
+namespace aspell { namespace sp {
 
   // WordInfo
 
   typedef unsigned int WordInfo; // 4 bits
 
-  enum CasePattern {Other, FirstUpper, AllLower, AllUpper};
+  //enum CasePattern {Other, FirstUpper, AllLower, AllUpper};
   //   Other      00
   //   FirstUpper 01
   //   AllLower   10
@@ -29,23 +29,17 @@ namespace aspeller {
   static const WordInfo ALL_PLAIN    = (1 << 2);
   static const WordInfo ALL_CLEAN    = (1 << 3);
 
-  using namespace acommon;
-
-  class Language;
+  class LangImpl;
   struct ConvertWord;
 
-  // WordEntry is an entry in the dictionary.  Both word and aff
-  // should point to a string that will stay in memory as long as the
-  // dictionary does unless under very special circumstances.  Thus
-  // free_ should generally be null.
+  // WordEntry is an entry in the dictionary.  
   struct WordEntry
   {
     const char * word;
     const char * aff;
     const char * catg;
     void (* adv_)(WordEntry *);
-    void (* free_)(WordEntry *);
-    void * intr[2];
+    void * intr[3];
     unsigned word_size;
     enum What {Other, Word, Soundslike, Clean, Misspelled} what;
     WordInfo word_info;
@@ -54,11 +48,11 @@ namespace aspeller {
     bool at_end() {return !word;}
     bool adv() {if (adv_) {adv_(this); return true;} word = 0; return false;}
     operator bool () const {return word != 0;}
-    OStream & write(OStream & o, const Language & l, Convert * c = 0) const;
+    OStream & write(OStream & o, const LangImpl & l, Convert * c = 0) const;
     WordEntry() {memset(this, 0, sizeof(WordEntry));}
-    void clear() {if (free_) free_(this); memset(this, 0, sizeof(WordEntry));}
-    ~WordEntry() {if (free_) free_(this);}
+    void clear() {memset(this, 0, sizeof(WordEntry));}
+    ~WordEntry() {}
   };
-}
+} }
 
 #endif
