@@ -12,13 +12,16 @@
    CURSES_INCLUDE_WORKAROUND_1
    CURSES_ONLY
    HAVE_GETCH
+   HAVE_MBLEN
    All these macros need to have a true value and not just be defined
 */
 
-#include <wchar.h>
-
 #include "settings.h"
 
+#ifdef DEFINE_XOPEN_SOURCE_EXTENDED
+#  define _XOPEN_SOURCE_EXTENDED 1
+#endif
+
 #ifdef CURSES_NON_POSIX
 #define CURSES_ONLY 1
 #endif
@@ -30,7 +33,9 @@
 #include "convert.hpp"
 #include "config.hpp"
 
-using namespace acommon;
+#include "gettext.h"
+
+using namespace aspell;
 
 StackPtr<CheckerString> state;
 const char * last_prompt = 0;
@@ -57,11 +62,7 @@ extern "C" {int getch();}
 
 #if HAVE_LIBCURSES
 
-#  if USE_NCURSESW
-#    include <ncursesw/curses.h>
-#  else
-#    include <curses.h>
-#  endif
+#include CURSES_HEADER
 
 #if CURSES_INCLUDE_STANDARD
 
@@ -102,6 +103,12 @@ static SCREEN * term;
 
 #endif // HAVE_LIBCURSES
 
+#ifdef HAVE_MBLEN
+#  include <wchar.h>
+#else
+#  define mblen(x,y) (*(x) ? 1 : 0)
+#endif
+
 static void cleanup (void) {
 #if   HAVE_LIBCURSES
   if (use_curses) {
@@ -375,9 +382,30 @@ void get_line(String & line) {
       }
       wrefresh(choice_w);
     }
+#ifdef HAVE_WIDE_CURSES
+    Vector<wchar_t> wstr;
+    for (int i = begin_x; i < end_x; ++i) {
+      cchar_t cc;
+      attr_t att;
+      short cp;
+      mvwin_wch(choice_w, 0, i, &cc);
+      size_t s = getcchar(&cc, 0, &att, &cp, 0);
+      wstr.resize(s+1); // +1 to allow room for the null character
+      getcchar(&cc, wstr.data(), &att, &cp, 0);
+      s = wcstombs(0, wstr.data(), 0);
+      if (s != (size_t)-1) {
+        int pos = line.alloc(s); // this always leaves space for the
+                                 // null character
+        wcstombs(line.data(pos), wstr.data(), s);
+      } else {
+        line += '?';
+      }
+    }
+#else
     for (int i = begin_x; i < end_x; ++i) {
       line += mvwinch(choice_w, 0, i) & A_CHARTEXT;
     }
+#endif
     menu_text = StdMenu;
     display_menu();
     doupdate();
@@ -412,8 +440,8 @@ void get_choice(int & c) {
       handle_last_signal();
       c0 = wgetch(choice_w);
     } while (c0 == ERR);
-    if (c == KEY_BREAK)
-      c = control('c');
+    if (c0 == KEY_BREAK)
+      c0 = control('c');
     if (1 <= c0 && c0 < 128) {
       c = static_cast<char>(c0);
       waddch(choice_w,c);
@@ -502,14 +530,25 @@ void display_misspelled_word() {
 
       wattrset(text_w,A_NORMAL);
       int last_space_pos = 0;
-      const char * last_space = i->begin();
+      const char * last_space = j;
+      int prev_glyph_pos = 0;
+      const char * prev_glyph = j;
 
       // NOTE: Combining characters after a character at the very end of
       //   a line will cause an unnecessary word wrap.  Unfortunately I
       //   do not know how to avoid it as they is no portable way to
       //   find out if the next character will combine with the
       //   previous.
-      while (j < i->end() && x0 <= x && *j != '\n')
+
+      // We check that:
+      // - we haven't reached the end of the text
+      // - we haven't reached the end of the line
+      // - curse haven't jumped to the next screen line
+      // - curse haven't reached the end of the screen
+      while (j < i->end()
+             && *j != '\n' 
+             && (y0 == y && x >= x0) 
+             && x0 < width - 1)
       {
         if (asc_isspace(*j)) {
           last_space_pos = x;
@@ -521,26 +560,39 @@ void display_misspelled_word() {
         } else if (j == word_end) {
           wattrset(text_w,A_NORMAL);
         }
-        int len = mbrlen(j, MB_CUR_MAX, NULL);
+        
+        int len = mblen(j, MB_CUR_MAX);
+        int res = OK;
         if (len > 0) {
-          waddnstr(text_w, j, len);
+          res = waddnstr(text_w, const_cast<char *>(j), len);
         } else {
           waddch(text_w, ' ');
           len = 1;
         }
+
         j += len;
         x0 = x;
         getyx(text_w, y, x);
+
+        if (x != x0 || res == ERR) {
+          prev_glyph_pos = x0;
+          prev_glyph = j - len;
+        }
+        
       }
       y = y0;
       if (j == i->end() || *j == '\n') {
         ++i;
         j = i->begin();
       } else {
-        if (x - last_space_pos < width/3) {
+        if (width - last_space_pos < width/3) {
           wmove(text_w, y, last_space_pos);
           wclrtoeol(text_w);
           j = last_space + 1;
+        } else {
+          wmove(text_w, y, prev_glyph_pos);
+          wclrtoeol(text_w);
+          j = prev_glyph;
         }
         wmove(text_w, y, width-1);
         wattrset(text_w,A_NORMAL);
@@ -581,7 +633,7 @@ static void print_truncate(FILE * out, const char * word, int width) {
   int i;
   int len = 0;
   for (i = 0; i < width-1 && *word; word += len, ++i) {
-    len = mbrlen(word, MB_CUR_MAX, NULL);
+    len = mblen(word, MB_CUR_MAX);
     if (len > 0) {
       put(out, word, len);
     } else {
@@ -645,8 +697,8 @@ static void print_truncate(WINDOW * out, const char * word, int width) {
   getyx(out, y, x);
   int stop = x + width - 1;
   while (x <= stop && *word) {
-    len = mbrlen(word, MB_CUR_MAX, NULL);
-    assert(len > 0);
+    len = mblen(word, MB_CUR_MAX);
+    if (len <= 0) len = 1;
     put(out, word, len);
     word += len;
     y0 = y;
